package com.benyun.project.third.area.service.impl;

import java.util.List;

import com.benyun.project.third.area.domain.OmsArea;
import com.benyun.project.third.area.mapper.OmsAreaMapper;
import com.benyun.project.third.area.service.IOmsAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.benyun.common.utils.text.Convert;

/**
 * 行政区划数据Service业务层处理
 *
 * @author benyun
 * @date 2023-12-14
 */
@Service
public class OmsAreaServiceImpl implements IOmsAreaService
{
    @Autowired
    private OmsAreaMapper omsAreaMapper;

    /**
     * 查询行政区划数据
     *
     * @param id 行政区划数据主键
     * @return 行政区划数据
     */
    @Override
    public OmsArea selectOmsAreaById(String id)
    {
        return omsAreaMapper.selectOmsAreaById(id);
    }

    /**
     * 查询行政区划数据列表
     *
     * @param omsArea 行政区划数据
     * @return 行政区划数据
     */
    @Override
    public List<OmsArea> selectOmsAreaList(OmsArea omsArea)
    {
        return omsAreaMapper.selectOmsAreaList(omsArea);
    }

    /**
     * 新增行政区划数据
     *
     * @param omsArea 行政区划数据
     * @return 结果
     */
    @Override
    public int insertOmsArea(OmsArea omsArea)
    {
        return omsAreaMapper.insertOmsArea(omsArea);
    }

    /**
     * 修改行政区划数据
     *
     * @param omsArea 行政区划数据
     * @return 结果
     */
    @Override
    public int updateOmsArea(OmsArea omsArea)
    {
        return omsAreaMapper.updateOmsArea(omsArea);
    }

    /**
     * 批量删除行政区划数据
     *
     * @param ids 需要删除的行政区划数据主键
     * @return 结果
     */
    @Override
    public int deleteOmsAreaByIds(String ids)
    {
        return omsAreaMapper.deleteOmsAreaByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除行政区划数据信息
     *
     * @param id 行政区划数据主键
     * @return 结果
     */
    @Override
    public int deleteOmsAreaById(String id)
    {
        return omsAreaMapper.deleteOmsAreaById(id);
    }
}
